
#include <allegro.h>
#include <stdio.h>

#include "window_mode2.h"



volatile int quit = 0;

void CloseWindow() {
   quit = 1;
}
END_OF_FUNCTION(CloseWindow);

/// filters for gfx_mode_select_filter based on graphics mode types
// return 0 to accept gfx mode, filter is passed these (card, w, h, color_depth)

int filter_non_windowed(int card , int w , int h , int color_depth) {
   if (get_gfx_mode_type(card) & GFX_TYPE_WINDOWED) {return 0;}
   return 1;
}

int filter_non_fullscreen(int card , int w , int h , int color_depth) {
   if (get_gfx_mode_type(card) & GFX_TYPE_FULLSCREEN) {return 0;}
   return 1;
}

int filter_non_definite(int card , int w , int h , int color_depth) {
   if (get_gfx_mode_type(card) & GFX_TYPE_DEFINITE) {return 0;}
   return 1;
}

int filter_magic_drivers(int card , int w , int h , int color_depth) {
   if (get_gfx_mode_type(card) & GFX_TYPE_MAGIC) {return 1;}
   return 0;
}

int filter_none(int card , int w , int h , int color_depth) {
   return 0;
}

const char* al_id_to_ascii(int al_id);
const char* graphics_card_name(int card);


int main() {

   const int dw = 640;
   const int dh = 480;
   const int gmode = GFX_AUTODETECT_WINDOWED;
   int dcd = 0;
   BITMAP* bg = 0;
   BITMAP* buffer = 0;
   int i = 0 , j = 0;

   #define NUM_TEXTLINES 11
   const char* text[NUM_TEXTLINES] = {
      "Press the listed key to test the option. The gfx_mode_select_ex",
      "function will be called using the specified filter as the callback.",
      "Each filter tests the return value of get_gfx_mode_type for specific types.",
      "This is only a test, so the graphics mode won't actually change.",
      "{ gfx_type = get_gfx_mode_type(card) }",
      "  1 - Keep windowed modes   (gfx_type & GFX_TYPE_WINDOWED  )",
      "  2 - Keep fullscreen modes (gfx_type & GFX_TYPE_FULLSCREEN)",
      "  3 - Keep definite modes   (gfx_type & GFX_TYPE_DEFINITE  )",
      "  4 - Reject magic modes    (gfx_type & GFX_TYPE_MAGIC     )",
      "  5 - Don't filter any modes. (For comparison)",
      "ESC - Close the program."
   };

   const char* messages[5] = {
      "Showing only windowed modes...",
      "Showing only fullscreen modes...",
      "Showing only definite modes...",
      "Showing only non magic driver modes...",
      "Not filtering any modes..."
   };

   /// For text positioning
   int vspace = 0 , linespacing = 0;
   int width = 0, maxwidth = 0;
   int height = 0;
   int textx = 0 , texty = 0;
   
   int msgx = 0 , msgy = 0;
   
   char mode_text[2][128];
   int mtx = 0 , mty = 0;

   int redraw = 1 , call_select = 0;

   int keyread = 0;

   /// For passing to gfx_mode_select_filter
   int card = 0 , w = 0 , h = 0 , color_depth = 0;

   typedef int(*FILTER_FUNC)(int,int,int,int);
   const FILTER_FUNC filter[5] = {
      filter_non_windowed,
      filter_non_fullscreen,
      filter_non_definite,
      filter_magic_drivers,
      filter_none
   };
   int filter_index = 0;

   /// Initialization
   if (allegro_init()     != 0) {return 0;}
   if (install_timer()    != 0) {return 0;}
   if (install_keyboard() != 0) {return 0;}
   if (install_mouse()     < 0) {return 0;}

   dcd = desktop_color_depth();
   if (dcd == 0) {dcd = 32;}

   set_color_depth(dcd);
   if (set_gfx_mode(gmode , dw , dh , 0 , 0) != 0) {return 0;}

   clear_to_color(screen , makecol(0,0,0));
   bg     = create_bitmap(dw , dh);
   buffer = create_bitmap(dw , dh);
   if (!bg || !buffer) {
      if (bg)     {destroy_bitmap(buffer);}
      if (buffer) {destroy_bitmap(buffer);}
   }
   clear_to_color(bg     , makecol(0,0,0));
   clear_to_color(buffer , makecol(0,0,0));

   LOCK_VARIABLE(quit);
   LOCK_FUNCTION(CloseWindow);
   set_close_button_callback(CloseWindow);

   card = gmode;
   w =  dw;
   h = dh;
   color_depth = dcd;

   vspace = text_height(font)/2;
   linespacing = text_height(font) + vspace;
   for (i = 0 ; i < NUM_TEXTLINES ; ++i) {
      width = text_length(font , text[i]);
      if (width > maxwidth) {maxwidth = width;}
   }
   height = NUM_TEXTLINES*text_height(font) + (NUM_TEXTLINES - 1)*vspace;
   textx = (dw - maxwidth)/2;
   texty = 2*linespacing;//(dh - height)/2;
   for (i = 0 ; i < NUM_TEXTLINES ; ++i) {
      textout_ex(bg , font , text[i] , textx , texty + i*linespacing , makecol(255,255,255) , -1);
   }


   msgx = dw/2;
   msgy = dh - (8*linespacing)/2;
   mtx = dw/2;
   mty = dh - (5*linespacing)/2;
   for (i = 0 ; i < 2 ; ++i) {
      for (j = 0 ; j < 128 ; ++j) {
         mode_text[i][j] = '\0';
      }
   }
   //int sprintf( char *buffer, const char *format, ... );
   sprintf((char*)&mode_text[0][0] , "Initial graphics mode set : %s" , graphics_card_name(gmode));
   sprintf((char*)&mode_text[1][0] , "Actual graphics mode set : %s" , graphics_card_name(get_gfx_mode()));
   
   textout_centre_ex(bg , font , (char*)&mode_text[0][0] , mtx , mty , makecol(255,255,255) , -1);
   textout_centre_ex(bg , font , (char*)&mode_text[1][0] , mtx , mty + linespacing , makecol(255,255,255) , -1);
   
   while(!quit) {
      
      if (keypressed()) {
         keyread = readkey() >> 8;
         switch (keyread) {
            case KEY_1 :
               filter_index = 0;
               call_select = 1;
               break;
            case KEY_2 :
               filter_index = 1;
               call_select = 1;
               break;
            case KEY_3 :
               filter_index = 2;
               call_select = 1;
               break;
            case KEY_4 :
               filter_index = 3;
               call_select = 1;
               break;
            case KEY_5 :
               filter_index = 4;
               call_select = 1;
               break;
            case KEY_ESC :
               quit = 1;
               break;
         }
         clear_keybuf();
         if (call_select) {
            textout_centre_ex(screen , font , messages[filter_index] , msgx , msgy , makecol(0,255,0) , -1);
            
            //int gfx_mode_select_filter(int *card, int *w, int *h, int *color_depth, int (*filter)(int, int, int, int));
            gfx_mode_select_filter(&card , &w , &h , &color_depth , filter[filter_index]);
            call_select = 0;
            redraw = 1;
         }
      }
      
      if (redraw) {
         blit(bg , screen , 0 , 0 , 0 , 0 , bg->w , bg->h);
         redraw = 0;
      }
      rest(20);
   }

   destroy_bitmap(bg);
   destroy_bitmap(buffer);
   return 0;
}
END_OF_MAIN()


const char* al_id_to_ascii(int al_id) {
   static char ascii_name[5] = {'\0','\0','\0','\0','\0'};
   ascii_name[0] = (char)((al_id >> 24) & 0xFF);
   ascii_name[1] = (char)((al_id >> 16) & 0xFF);
   ascii_name[2] = (char)((al_id >> 8) & 0xFF);
   ascii_name[3] = (char)(al_id & 0xFF);
   return (const char*)ascii_name;
}

/// gfx.h lines 34-39
// #define GFX_TEXT                       -1
// #define GFX_AUTODETECT                 0
// #define GFX_AUTODETECT_FULLSCREEN      1
// #define GFX_AUTODETECT_WINDOWED        2
// #define GFX_SAFE                       AL_ID('S','A','F','E')
// #define GFX_NONE                       AL_ID('N','O','N','E')
const char* graphics_card_name(int card) {
   int i = 0;
   static char name[26];
   for (i = 0 ; i < 26 ; ++i) {name[i] = '\0';}
   switch (card) {
      case GFX_TEXT                  : strncpy(name , "GFX_TEXT"                  , 25);break;
      case GFX_AUTODETECT            : strncpy(name , "GFX_AUTODETECT"            , 25);break;
      case GFX_AUTODETECT_FULLSCREEN : strncpy(name , "GFX_AUTODETECT_FULLSCREEN" , 25);break;
      case GFX_AUTODETECT_WINDOWED   : strncpy(name , "GFX_AUTODETECT_WINDOWED"   , 25);break;
      case GFX_SAFE                  : strncpy(name , "GFX_SAFE"                  , 25);break;
      case GFX_NONE                  : strncpy(name , "GFX_NONE"                  , 25);break;
      default : return al_id_to_ascii(card);
   }
   return (const char*)name;
}


